@@ -20,4 +20,4 @@
  */
 
 #define CATCH_CONFIG_MAIN  // This tells Catch to provide a main()
-#include "catch.hpp"
\ No newline at end of file
+#include "catch2/catch.hpp"
\ No newline at end of file
