@@ -1,8 +1,28 @@
+/*
+ * Copyright (c) 2017-2018 Nitrokey UG
+ *
+ * This file is part of Nitrokey App.
+ *
+ * Nitrokey App is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * any later version.
+ *
+ * Nitrokey App is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with Nitrokey App. If not, see <http://www.gnu.org/licenses/>.
+ *
+ * SPDX-License-Identifier: GPL-3.0
+ */
 
 #include "StorageActions.h"
 #include "src/ui/pindialog.h"
 #include "src/utils/bool_values.h"
-#include "libnitrokey/include/NitrokeyManager.h"
+#include <libnitrokey/NitrokeyManager.h>
 
 #ifdef Q_OS_LINUX
 #include "systemutils.h"
@@ -19,10 +39,11 @@
 #define LOCAL_PASSWORD_SIZE 40
 #include <memory>
 #include <src/core/ThreadWorker.h>
-#include <libnitrokey/include/stick20_commands.h>
+#include <libnitrokey/stick20_commands.h>
 
 
 void unmountEncryptedVolumes() {
+    return;
 #if defined(Q_OS_LINUX)
   std::string endev = systemutils::getEncryptedDevice();
   if (endev.size() < 1)
@@ -64,7 +85,7 @@ void StorageActions::startStick20EnableCryptedVolume() {
       return;
   }
 
-  PinDialog dialog(PinDialog::USER_PIN, nullptr);
+  PinDialog dialog(PinType::USER_PIN, nullptr);
 
   const auto user_wants_to_proceed = QDialog::Accepted == dialog.exec();
   if (user_wants_to_proceed) {
@@ -154,8 +175,12 @@ void StorageActions::startStick20DisableCryptedVolume() {
         CryptedVolumeActive = false;
         emit storageStatusChanged();
         show_message_function(tr("Encrypted volume disabled"));
-      }
-       else {
+        //  for v0.50 and below ask for reinsertion to complete the lock procedure
+        if (libada::i()->getMinorFirmwareVersion() <= 50){
+          csApplet()->messageBox(
+              tr("To complete the lock procedure, please remove and reconnect the Nitrokey."));
+        }
+      } else {
         csApplet()->warningBox(tr("Could not lock encrypted volume.") + " "
                                + tr("Status code: %1").arg(data["error"].toInt())); //FIXME use existing translation
       }
@@ -177,7 +202,7 @@ void StorageActions::startStick20EnableHiddenVolume() {
   if (!user_wants_to_proceed)
     return;
 
-  PinDialog dialog(PinDialog::HIDDEN_VOLUME, nullptr);
+  PinDialog dialog(PinType::HIDDEN_VOLUME, nullptr);
   const auto user_gives_password = dialog.exec() == QDialog::Accepted ;
 
   if (!user_gives_password) {
@@ -320,6 +345,11 @@ void StorageActions::startLockDeviceAction(bool ask_for_confirmation) {
         CryptedVolumeActive = false;
         emit storageStatusChanged();
         show_message_function(tr("Device locked")); //FIXME use existing translation
+        //  for v0.50 and below ask for reinsertion to complete the lock procedure
+        if (libada::i()->getMinorFirmwareVersion() <= 50){
+          csApplet()->messageBox(
+              tr("To complete the lock procedure, please remove and reconnect the Nitrokey."));
+        }
       }
       else {
         csApplet()->warningBox(tr("Could not lock device.") + " "
@@ -341,7 +371,23 @@ void StorageActions::startStick20EnableFirmwareUpdate() {
     return;
   }
 
-  PinDialog dialog(PinDialog::FIRMWARE_PIN);
+  auto successMessage = tr("Device set in update mode");
+  auto failureMessage = tr("Device could not be set in update mode.");
+
+  bool success = false;
+  // try with default password
+  // ask for password when fails
+  runAndHandleErrorsInUI(successMessage, "",
+                         [&success](){ //FIXME use secure string
+                           local_sync();
+                           auto m = nitrokey::NitrokeyManager::instance();
+                           m->enable_firmware_update("12345678");
+                           success = true;
+                         },[](){});
+
+  if(success) return;
+
+  PinDialog dialog(PinType::FIRMWARE_PIN);
   user_wants_to_proceed = QDialog::Accepted == dialog.exec();
 
   if (!user_wants_to_proceed) {
@@ -349,8 +395,6 @@ void StorageActions::startStick20EnableFirmwareUpdate() {
   }
   auto s = dialog.getPassword();
 
-  auto successMessage = tr("Device set in update mode");
-  auto failureMessage = tr("Device could not be set in update mode.");
   runAndHandleErrorsInUI(successMessage, failureMessage,
      [s](){ //FIXME use secure string
          local_sync();
@@ -361,7 +405,7 @@ void StorageActions::startStick20EnableFirmwareUpdate() {
 
 
 void StorageActions::startStick20ExportFirmwareToFile() {
-  PinDialog dialog(PinDialog::ADMIN_PIN);
+  PinDialog dialog(PinType::ADMIN_PIN);
   bool user_provided_PIN = QDialog::Accepted == dialog.exec();
   if (!user_provided_PIN) {
     return;
@@ -379,10 +423,10 @@ void StorageActions::startStick20DestroyCryptedVolume(int fillSDWithRandomChars)
   bool user_entered_PIN;
   bool answer;
 
-  answer = csApplet()->yesOrNoBox(tr("WARNING: Generating new AES keys will destroy the encrypted volumes, "
+  answer = (fillSDWithRandomChars == 1) || csApplet()->yesOrNoBox(tr("WARNING: Generating new AES keys will destroy the encrypted volumes, "
                                          "hidden volumes, and password safe! Continue?"), false);
   if (answer) {
-    PinDialog dialog(PinDialog::ADMIN_PIN);
+    PinDialog dialog(PinType::ADMIN_PIN);
     user_entered_PIN = QDialog::Accepted == dialog.exec();
     if (!user_entered_PIN) {
       return;
@@ -478,7 +522,7 @@ void StorageActions::_execute_SD_clearing(const std::string &s) {
 }
 
 void StorageActions::startStick20FillSDCardWithRandomChars() {
-  PinDialog dialog(PinDialog::ADMIN_PIN);
+  PinDialog dialog(PinType::ADMIN_PIN);
   bool user_provided_PIN = QDialog::Accepted == dialog.exec();
   if (user_provided_PIN) {
     auto s = dialog.getPassword();
@@ -497,24 +541,28 @@ void StorageActions::runAndHandleErrorsInUI(QString successMessage, QString oper
     show_message_function(successMessage);
   }
   catch (CommandFailedException &e){
-    if (e.reason_wrong_password()){
-      csApplet()->warningBox(operationFailureMessage + " " //FIXME use existing translation
-                             + tr("Wrong password."));
-    } else {
-      csApplet()->warningBox(operationFailureMessage + " "
-                             + tr("Status code: %1").arg(e.last_command_status)); //FIXME use existing translation
+    if (!operationFailureMessage.isEmpty()){
+      if (e.reason_wrong_password()){
+        csApplet()->warningBox(operationFailureMessage + " "
+                               + tr("Wrong password."));
+      } else {
+        csApplet()->warningBox(operationFailureMessage + " "
+                               + tr("Status code: %1").arg(e.last_command_status));
+      }
     }
   }
   catch (DeviceCommunicationException &e){
-    csApplet()->warningBox(operationFailureMessage + " " +
-                           tr("Communication issue.")); //FIXME use existing translation
+    if (!operationFailureMessage.isEmpty()){
+      csApplet()->warningBox(operationFailureMessage + " " +
+                           tr("Communication issue."));
+    }
   }
 //  end_progress_function();
 }
 
 
 void StorageActions::startStick20ClearNewSdCardFound() {
-  PinDialog dialog(PinDialog::ADMIN_PIN);
+  PinDialog dialog(PinType::ADMIN_PIN);
 
   bool user_provided_pin = QDialog::Accepted == dialog.exec();
   if (!user_provided_pin) {
@@ -535,7 +583,12 @@ void StorageActions::startStick20ClearNewSdCardFound() {
 
 
 void StorageActions::startStick20SetReadOnlyUncryptedVolume() {
-  PinDialog dialog(PinDialog::USER_PIN);
+  using nm = nitrokey::NitrokeyManager;
+  auto type = PinType::ADMIN_PIN;
+  if(nm::instance()->set_unencrypted_volume_rorw_pin_type_user()){
+    type = PinType::USER_PIN;
+  }
+  PinDialog dialog(type);
 
   bool user_provided_pin = QDialog::Accepted == dialog.exec();
   if (!user_provided_pin) {
@@ -546,9 +599,19 @@ void StorageActions::startStick20SetReadOnlyUncryptedVolume() {
   auto operationSuccessMessage = tr("Unencrypted volume set read-only"); //FIXME use existing translation
   auto s = dialog.getPassword();
 
-  runAndHandleErrorsInUI(operationSuccessMessage, operationFailureMessage, [s]() {
+  runAndHandleErrorsInUI(operationSuccessMessage, operationFailureMessage, [s, type]() {
     auto m = nitrokey::NitrokeyManager::instance();
-    m->set_unencrypted_read_only(s.c_str()); //FIXME use secure string
+    //FIXME use secure string
+    switch(type){
+      case USER_PIN:
+        m->set_unencrypted_read_only(s.c_str());
+        break;
+      case ADMIN_PIN:
+        m->set_unencrypted_read_only_admin(s.c_str());
+        break;
+      default:
+        break;
+    }
   }, [this]() {
     emit storageStatusChanged();
   });
@@ -556,7 +619,12 @@ void StorageActions::startStick20SetReadOnlyUncryptedVolume() {
 }
 
 void StorageActions::startStick20SetReadWriteUncryptedVolume() {
-  PinDialog dialog(PinDialog::USER_PIN);
+  using nm = nitrokey::NitrokeyManager;
+  auto type = PinType::ADMIN_PIN;
+  if(nm::instance()->set_unencrypted_volume_rorw_pin_type_user()){
+    type = PinType::USER_PIN;
+  }
+  PinDialog dialog(type);
 
   bool user_provided_pin = QDialog::Accepted == dialog.exec();
   if (!user_provided_pin) {
@@ -567,9 +635,61 @@ void StorageActions::startStick20SetReadWriteUncryptedVolume() {
   auto operationSuccessMessage = tr("Unencrypted volume set read-write"); //FIXME use existing translation
   auto s = dialog.getPassword();
 
+  runAndHandleErrorsInUI(operationSuccessMessage, operationFailureMessage, [s, type]() {
+    auto m = nitrokey::NitrokeyManager::instance();
+    //FIXME use secure string
+    switch(type){
+      case USER_PIN:
+        m->set_unencrypted_read_write(s.c_str());
+        break;
+      case ADMIN_PIN:
+        m->set_unencrypted_read_write_admin(s.c_str());
+        break;
+      default:
+        break;
+    }
+  }, [this]() {
+    emit storageStatusChanged();
+  });
+
+}
+
+void StorageActions::startStick20SetReadOnlyEncryptedVolume() {
+  PinDialog dialog(PinType::ADMIN_PIN);
+
+  bool user_provided_pin = QDialog::Accepted == dialog.exec();
+  if (!user_provided_pin) {
+    return;
+  }
+
+  auto operationFailureMessage = tr("Cannot set encrypted volume read-only"); //FIXME use existing translation
+  auto operationSuccessMessage = tr("Encrypted volume set read-only"); //FIXME use existing translation
+  auto s = dialog.getPassword();
+
   runAndHandleErrorsInUI(operationSuccessMessage, operationFailureMessage, [s]() {
     auto m = nitrokey::NitrokeyManager::instance();
-    m->set_unencrypted_read_write(s.c_str()); //FIXME use secure string
+    m->set_encrypted_volume_read_only(s.c_str()); //FIXME use secure string
+  }, [this]() {
+    emit storageStatusChanged();
+  });
+
+}
+
+void StorageActions::startStick20SetReadWriteEncryptedVolume() {
+  PinDialog dialog(PinType::ADMIN_PIN);
+
+  bool user_provided_pin = QDialog::Accepted == dialog.exec();
+  if (!user_provided_pin) {
+    return;
+  }
+
+  auto operationFailureMessage = tr("Cannot set encrypted volume read-write"); //FIXME use existing translation
+  auto operationSuccessMessage = tr("Encrypted volume set read-write"); //FIXME use existing translation
+  auto s = dialog.getPassword();
+
+  runAndHandleErrorsInUI(operationSuccessMessage, operationFailureMessage, [s]() {
+    auto m = nitrokey::NitrokeyManager::instance();
+    m->set_encrypted_volume_read_write(s.c_str()); //FIXME use secure string
   }, [this]() {
     emit storageStatusChanged();
   });
@@ -583,7 +703,7 @@ void StorageActions::startStick20LockStickHardware() {
   stick20LockFirmwareDialog firmwareDialog(nullptr);
   bool user_wants_to_proceed = QDialog::Accepted == firmwareDialog.exec();
   if (user_wants_to_proceed) {
-    PinDialog pinDialog(PinDialog::ADMIN_PIN);
+    PinDialog pinDialog(PinType::ADMIN_PIN);
 
     bool user_provided_PIN = pinDialog.exec() == QDialog::Accepted;
     if (!user_provided_PIN) {
