@@ -69,6 +69,11 @@ Tray::~Tray() {
   destroyThread();
 }
 
+void Tray::delayedShowIndicator(){
+  if(!trayIcon->isSystemTrayAvailable()) return;
+  trayIcon->show();
+  delayedShowTimer->stop();
+}
 
 /*
  * Create the tray menu.
@@ -79,7 +84,10 @@ void Tray::createIndicator() {
   connect(trayIcon, SIGNAL(activated(QSystemTrayIcon::ActivationReason)), this,
           SLOT(iconActivated(QSystemTrayIcon::ActivationReason)));
 
-  trayIcon->show();
+  delayedShowTimer = new QTimer(this);
+  connect(delayedShowTimer, SIGNAL(timeout()), this, SLOT(delayedShowIndicator()));
+  delayedShowTimer->setSingleShot(false);
+  delayedShowTimer->start(2000);
 
   // Initial message
   if (debug_mode)
@@ -394,6 +402,7 @@ void Tray::initActionsForStick20() {
           SLOT(startResetUserPassword()));
 
   LockDeviceAction = new QAction(tr("&Lock Device"), main_window);
+  LockDeviceAction->setIcon(GraphicsTools::loadColorize(":/images/new/icon_unsafe.svg"));
   connect(LockDeviceAction, SIGNAL(triggered()), main_window, SLOT(startLockDeviceAction()));
 
   Stick20ActionUpdateStickStatus = new QAction(tr("Smartcard or SD card are not ready"), main_window);
@@ -476,6 +485,10 @@ void Tray::populateOTPPasswordMenu() {
   //should not run before worker is done
   QMutexLocker mutexLocker(&worker->mtx);
 
+  if (!trayMenuPasswdSubMenu->actions().empty()) {
+    return;
+  }
+
 
   for (int i=0; i < TOTP_SLOT_COUNT; i++){
     auto slotName = libada::i()->getTOTPSlotName(i);
@@ -530,9 +543,6 @@ void Tray::generateMenuForProDevice() {
     trayMenuSubConfigure = trayMenu->addMenu(tr("Configure"));
     trayMenuSubConfigure->setIcon(GraphicsTools::loadColorize(":/images/new/icon_settings.svg"));
 
-    trayMenuSubConfigure_tray = trayMenu->addMenu(tr("Configure"));
-    trayMenuSubConfigure_tray->setIcon(GraphicsTools::loadColorize(":/images/new/icon_settings.svg", true));
-
     if (TRUE == libada::i()->isPasswordSafeAvailable())
       trayMenuSubConfigure->addAction(configureActionStick20);
     else
