@@ -28,8 +28,10 @@
 #include <vector>
 #include <string>
 
+#define TO_BASE32_LEN(x)  ((x)/10*16)
+
 static const unsigned int SECRET_LENGTH = 40;
-static const unsigned int SECRET_LENGTH_BASE32 = SECRET_LENGTH / 10 * 16;
+static const unsigned int SECRET_LENGTH_BASE32 = TO_BASE32_LEN(SECRET_LENGTH);
 static const unsigned int SECRET_LENGTH_HEX = SECRET_LENGTH * 2;
 
 std::vector<uint8_t> decodeBase32Secret(const std::string secret, const bool debug_mode = false);
