@@ -1,20 +1,23 @@
 /*
  * Author: Copyright (C)  Rudolf Boeddeker  Date: 2014-04-12
+ * Copyright (c) 2014-2018 Nitrokey UG
  *
- * This file is part of Nitrokey.
+ * This file is part of Nitrokey App.
  *
- * Nitrokey is free software: you can redistribute it and/or modify
+ * Nitrokey App is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation, either version 3 of the License, or
  * any later version.
  *
- * Nitrokey is distributed in the hope that it will be useful,
+ * Nitrokey App is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with Nitrokey. If not, see <http://www.gnu.org/licenses/>.
+ * along with Nitrokey App. If not, see <http://www.gnu.org/licenses/>.
+ *
+ * SPDX-License-Identifier: GPL-3.0
  */
 
 
@@ -23,8 +26,8 @@
 #include "ui_aboutdialog.h"
 
 #include "libada.h"
-#include "libnitrokey/include/NitrokeyManager.h"
-#include "libnitrokey/include/DeviceCommunicationExceptions.h"
+#include <libnitrokey/NitrokeyManager.h>
+#include <libnitrokey/DeviceCommunicationExceptions.h>
 using nm = nitrokey::NitrokeyManager;
 
 using namespace AboutDialogUI;
@@ -33,7 +36,7 @@ static const int invalid_value = 99;
 
 AboutDialog::AboutDialog(QWidget *parent)
     : QDialog(parent), ui(nullptr) {
-  ui = std::make_shared<Ui::AboutDialog>();
+  ui = new Ui::AboutDialog;
   ui->setupUi(this);
 
   connect(&worker_thread, SIGNAL(started()), &worker, SLOT(fetch_device_data()));
@@ -41,17 +44,13 @@ AboutDialog::AboutDialog(QWidget *parent)
   worker.moveToThread(&worker_thread);
   worker_thread.start();
 
-  QPixmap image(":/images/splash.png");
-  QPixmap small_img = image.scaled(346, 80, Qt::KeepAspectRatio, Qt::FastTransformation);
-  QPixmap warning(":/images/warning.png");
-//  QPixmap small_warning = warning.scaled(50, 50, Qt::KeepAspectRatio, Qt::FastTransformation);
-  QPixmap info_img(":/images/info-icon.png");
-  QPixmap small_info = info_img.scaled(15, 15, Qt::KeepAspectRatio, Qt::FastTransformation);
+  QPixmap warning_orig(":/images/new/icon_warning.svg");
+  QPixmap warning = warning_orig.scaled(66, 58, Qt::KeepAspectRatio, Qt::SmoothTransformation);
+  QPixmap info_img(":/images/new/icon_info.svg");
+  QPixmap small_info = info_img.scaled(15, 15, Qt::KeepAspectRatio, Qt::SmoothTransformation);
 
   ui->info_icon->setPixmap(small_info);
   ui->warning_sign->setPixmap(warning);
-  ui->IconLabel->setPixmap(small_img);
-
 
   ui->admin_retry_label->setDisabled(true);
   ui->user_retry_label->setDisabled(true);
@@ -67,6 +66,8 @@ AboutDialog::AboutDialog(QWidget *parent)
   ui->not_erased_sd_label->hide();
   ui->newsd_label->hide();
 
+  ui->AboutLabel_2->setText(ui->AboutLabel_2->text().arg(COPYRIGHT_YEARS));
+
   if (libada::i()->isDeviceConnected()) {
     if (libada::i()->isStorageDeviceConnected()) {
       showStick20Configuration();
@@ -80,11 +81,13 @@ AboutDialog::AboutDialog(QWidget *parent)
   fixWindowGeometry();
 
   ManageWindow::bringToFocus(this);
+  connect(parent, SIGNAL(LongOperationStart()), this, SLOT(reject()));
 }
 
 AboutDialog::~AboutDialog() {
   worker_thread.quit();
   worker_thread.wait();
+  delete ui;
 }
 
 void AboutDialog::on_ButtonOK_clicked() { done(TRUE); }
@@ -93,7 +96,7 @@ void AboutDialog::on_ButtonOK_clicked() { done(TRUE); }
 #include <QTextStream>
 #include <QWindow>
 #include <QTextEdit>
-#include <libnitrokey/include/stick20_commands.h>
+#include <libnitrokey/stick20_commands.h>
 #include "licensedialog.h"
 
 
@@ -238,6 +241,7 @@ void Worker::fetch_device_data() {
   }
 
   try{
+    devdata.testFirmwareVersion = 0;
     devdata.majorFirmwareVersion = libada::i()->getMajorFirmwareVersion();
     devdata.minorFirmwareVersion = libada::i()->getMinorFirmwareVersion();
 
@@ -245,12 +249,14 @@ void Worker::fetch_device_data() {
     if (devdata.storage.active ) {
       devdata.storage.sdcard.size_GB = libada::i()->getStorageSDCardSizeGB();
       auto st = nm::instance()->get_status_storage(); //FIXME use libada?
+      devdata.testFirmwareVersion = st.versionInfo.build_iteration;
+      devdata.minorFirmwareVersion = st.versionInfo.minor;
       devdata.storage.volume_active.plain = st.VolumeActiceFlag_st.unencrypted;
       devdata.storage.volume_active.encrypted = st.VolumeActiceFlag_st.encrypted;
       devdata.storage.volume_active.hidden = st.VolumeActiceFlag_st.hidden;
-      devdata.storage.volume_active.plain_RW = st.ReadWriteFlagUncryptedVolume_u8;
-      devdata.storage.volume_active.encrypted_RW = st.ReadWriteFlagCryptedVolume_u8;
-      devdata.storage.volume_active.hidden_RW = st.ReadWriteFlagHiddenVolume_u8;
+      devdata.storage.volume_active.plain_RW = !st.ReadWriteFlagUncryptedVolume_u8;  //FIXME correct variable naming in lib
+      devdata.storage.volume_active.encrypted_RW = !st.ReadWriteFlagCryptedVolume_u8;
+      devdata.storage.volume_active.hidden_RW = !st.ReadWriteFlagHiddenVolume_u8;
       devdata.storage.keys_initiated = !st.StickKeysNotInitiated;
       devdata.storage.sdcard.is_new =  st.NewSDCardFound_st.NewCard; // st.NewSDCardFound_u8;
       devdata.storage.sdcard.filled_with_random = st.SDFillWithRandomChars_u8;
@@ -276,6 +282,7 @@ void Worker::fetch_device_data() {
   emit finished(true);
 }
 
+#include "src/ui/nitrokey-applet.h"
 
 void AboutDialog::update_device_slots(bool connected) {
   fixWindowGeometry();
@@ -291,6 +298,11 @@ void AboutDialog::update_device_slots(bool connected) {
 
   if (worker.devdata.storage.long_operation.status){
     OutputText.append(QString(tr("      *** Clearing data in progress ***")).append("\n"));
+    QApplication::processEvents();
+    reject();
+    QApplication::processEvents();
+    csApplet()->warningBox(tr("Cannot open dialog: ") + tr("      *** Clearing data in progress ***").trimmed());
+    return;
   }
 
   if (worker.devdata.comm_error) {
@@ -325,15 +337,25 @@ void AboutDialog::update_device_slots(bool connected) {
     ui->newsd_label->setVisible(worker.devdata.storage.sdcard.is_new);
     ui->not_erased_sd_label->setVisible(!worker.devdata.storage.sdcard.filled_with_random);
 
-    auto read_write_active = worker.devdata.storage.volume_active.plain_RW == 0; //FIXME correct variable naming in lib
+    auto read_write_active = worker.devdata.storage.volume_active.plain_RW;
     ui->unencrypted_volume_label->setText(read_write_active ? tr("READ/WRITE") : tr("READ ONLY"));
 
     if (worker.devdata.storage.volume_active.hidden) {
-      ui->hidden_volume_label->setText(tr("Active"));
+      ui->encrypted_volume_label->setText(
+          tr("Active") + " " + tr("(hidden)"));
+      ui->hidden_volume_label->setText(
+          worker.devdata.storage.volume_active.hidden_RW ?
+          tr("READ/WRITE") : tr("READ ONLY"));
     } else {
       ui->hidden_volume_label->setText(tr("Not active"));
-      ui->encrypted_volume_label->setText(
-          worker.devdata.storage.volume_active.encrypted ? tr("Active") : tr("Not active"));
+      if(!worker.devdata.storage.volume_active.encrypted){
+        ui->encrypted_volume_label->setText(tr("Not active"));
+      }
+      else {
+        ui->encrypted_volume_label->setText(
+          worker.devdata.storage.volume_active.encrypted_RW ?
+          tr("READ/WRITE") : tr("READ ONLY"));
+      }
     }
 
     ui->sd_id_label->setText(
@@ -345,12 +367,15 @@ void AboutDialog::update_device_slots(bool connected) {
   }
   ui->firmwareLabel->setText(QString::number(worker.devdata.majorFirmwareVersion)
                                  .append(".")
-                                 .append(QString::number(worker.devdata.minorFirmwareVersion)));
+                                 .append(QString::number(worker.devdata.minorFirmwareVersion))
+                             .append( worker.devdata.testFirmwareVersion != 0 ?
+                                      QString::number(worker.devdata.testFirmwareVersion) : "")
+  );
 
   ui->serialEdit->setText(QString::fromStdString(worker.devdata.cardSerial).trimmed());
 
 
-  if (0 == worker.devdata.storage.smartcard_id) {
+  if (0 == worker.devdata.storage.smartcard_id && worker.devdata.storage.active) {
     ui->serialEdit->setText("-");
     OutputText.append(QString(tr("\nSmartcard is not accessible\n\n")));
   }
