@@ -1,20 +1,23 @@
 /*
  * Author: Copyright (C) Rudolf Boeddeker  Date: 2013-08-13
+ * Copyright (c) 2013-2018 Nitrokey UG
  *
- * This file is part of Nitrokey 2
+ * This file is part of Nitrokey App.
  *
- * Nitrokey 2  is free software: you can redistribute it and/or modify
+ * Nitrokey App is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation, either version 3 of the License, or
  * any later version.
  *
- * Nitrokey is distributed in the hope that it will be useful,
+ * Nitrokey App is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with Nitrokey. If not, see <http://www.gnu.org/licenses/>.
+ * along with Nitrokey App. If not, see <http://www.gnu.org/licenses/>.
+ *
+ * SPDX-License-Identifier: GPL-3.0
  */
 
 #include "stick20changepassworddialog.h"
@@ -22,8 +25,9 @@
 #include "stick20responsedialog.h"
 #include "ui_stick20changepassworddialog.h"
 #include "libada.h"
-#include "libnitrokey/include/NitrokeyManager.h"
+#include <libnitrokey/NitrokeyManager.h>
 #include "src/core/ThreadWorker.h"
+#include <QStyle>
 using nm = nitrokey::NitrokeyManager;
 
 DialogChangePassword::DialogChangePassword(QWidget *parent, PasswordKind _kind):
@@ -46,6 +50,7 @@ DialogChangePassword::DialogChangePassword(QWidget *parent, PasswordKind _kind):
   fixWindowGeometry();
   ui->lineEdit_OldPW->setFocus();
   setModal(true);
+  connect(parent, SIGNAL(LongOperationStart()), this, SLOT(reject()));
 }
 
 DialogChangePassword::~DialogChangePassword() { delete ui; }
@@ -135,29 +140,29 @@ void DialogChangePassword::InitData(void) {
     case PasswordKind::USER:
       this->setWindowTitle(tr("Set User PIN"));
       ui->label_2->setText(tr("Current User PIN:"));
-      ui->label_3->setText(tr("New User PIN:"));
-      ui->label_4->setText(tr("New User PIN:"));
+      ui->label_3->setText(tr("New PIN:"));
+      ui->label_4->setText(tr("Confirm New PIN:"));
       minimumPasswordLength = minimumPasswordLengthUser;
       break;
     case PasswordKind::ADMIN:
       this->setWindowTitle(tr("Set Admin PIN"));
       ui->label_2->setText(tr("Current Admin PIN:"));
-      ui->label_3->setText(tr("New Admin PIN:"));
-      ui->label_4->setText(tr("New Admin PIN:"));
+      ui->label_3->setText(tr("New PIN:"));
+      ui->label_4->setText(tr("Confirm New PIN:"));
       minimumPasswordLength = minimumPasswordLengthAdmin;
       break;
     case PasswordKind::RESET_USER:
       this->setWindowTitle(tr("Reset User PIN"));
-      ui->label_2->setText(tr("Admin PIN:"));
+      ui->label_2->setText(tr("Current Admin PIN:"));
       ui->label_3->setText(tr("New User PIN:"));
-      ui->label_4->setText(tr("New User PIN:"));
+      ui->label_4->setText(tr("Confirm New User PIN:"));
       minimumPasswordLength = minimumPasswordLengthUser;
       break;
     case PasswordKind::UPDATE:
       this->setWindowTitle(tr("Change Firmware Password"));
       ui->label_2->setText(tr("Current Firmware Password:"));
-      ui->label_3->setText(tr("New Firmware Password:"));
-      ui->label_4->setText(tr("New Firmware Password:"));
+      ui->label_3->setText(tr("New Password:"));
+      ui->label_4->setText(tr("Confirm New Password:"));
       ui->checkBox->setText(tr("Show password"));
       minimumPasswordLength = minimumPasswordLengthFirmware;
       break;
@@ -170,11 +175,17 @@ void DialogChangePassword::InitData(void) {
   // replace %1 and %2 from text with proper values
   //(min and max of password length)
   QString text = ui->label_additional_information->text();
-  text = text.arg(minimumPasswordLength).arg(STICK20_PASSOWRD_LEN).arg(minimumPasswordLengthAdmin);
+
+  text = text.arg(minimumPasswordLength).arg(STICK20_PASSOWRD_LEN);
+
   if (kind == PasswordKind::UPDATE) {
-    text += QString("<i>") + tr("Once the firmware password is forgotten the Nitrokey can't be "
-                                    "updated or reset. Don't lose your firmware password, please.") +
-            QString("</i>");
+    text = "<i> " + tr("The Firmware password doesn’t have a retry counter, and therefore doesn’t prevent against password guessing attacks. "
+                       "A secure and complex password should be created with the use of: "
+                       "lower and upper case letters, numbers and special characters; with a length between %2 and %3 characters."
+                     "<br/>Default firmware password is: '%1'.").arg(defaultFirmwarePassword).arg(minimumPasswordLengthFirmware).arg(maximumPasswordLengthFirmware)
+            + "</i>";
+    text += QString("<p><b>") + tr("WARNING! If you lose your Firmware password, Nitrokey can’t be updated or reset!") + QString("</b></p>");
+
   }
   ui->label_additional_information->setText(text);
   fixWindowGeometry();
