@@ -1,22 +1,26 @@
 /*
  * Author: Copyright (C) Rudolf Boeddeker  Date: 2013-08-13
+ * Copyright (c) 2013-2018 Nitrokey UG
  *
- * This file is part of Nitrokey 2
+ * This file is part of Nitrokey App.
  *
- * Nitrokey 2  is free software: you can redistribute it and/or modify
+ * Nitrokey App is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation, either version 3 of the License, or
  * any later version.
  *
- * Nitrokey is distributed in the hope that it will be useful,
+ * Nitrokey App is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with Nitrokey. If not, see <http://www.gnu.org/licenses/>.
+ * along with Nitrokey App. If not, see <http://www.gnu.org/licenses/>.
+ *
+ * SPDX-License-Identifier: GPL-3.0
  */
 
+
 #ifndef DIALOGCHANGEPASSWORD_H
 #define DIALOGCHANGEPASSWORD_H
 
@@ -39,7 +43,9 @@ class DialogChangePassword : public QDialog {
   // FIXME extract constant to global config
     const static int minimumPasswordLengthUser = 6;
     const static int minimumPasswordLengthAdmin = 8;
-    const static int minimumPasswordLengthFirmware = 6;
+    const static int minimumPasswordLengthFirmware = 8;
+    const static int maximumPasswordLengthFirmware = 20;
+    static constexpr auto defaultFirmwarePassword = "12345678";
     int minimumPasswordLength = {};
 
   explicit DialogChangePassword(QWidget *parent, PasswordKind _kind);
