@@ -1,24 +1,27 @@
 /*
- * Author: Copyright (C) Rudolf Boeddeker  Date: 2014-04-13
+ * Author: Copyright (C) Rudolf Boeddeker  Date: 2013-08-13
+ * Copyright (c) 2013-2018 Nitrokey UG
  *
- * This file is part of Nitrokey 2
+ * This file is part of Nitrokey App.
  *
- * Nitrokey 2  is free software: you can redistribute it and/or modify
+ * Nitrokey App is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation, either version 3 of the License, or
  * any later version.
  *
- * Nitrokey is distributed in the hope that it will be useful,
+ * Nitrokey App is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with Nitrokey. If not, see <http://www.gnu.org/licenses/>.
+ * along with Nitrokey App. If not, see <http://www.gnu.org/licenses/>.
+ *
+ * SPDX-License-Identifier: GPL-3.0
  */
 
 #include <src/core/ThreadWorker.h>
-#include <libnitrokey/include/NitrokeyManager.h>
+#include <libnitrokey/NitrokeyManager.h>
 #include "stick20hiddenvolumedialog.h"
 #include "math.h"
 #include "mcvs-wrapper.h"
